@@ -22,6 +22,7 @@
 #include <ctype.h>
 #include "defs.h"
 #include "csv_util.h"
+#include "grtcirc.h"
 
 #define MYNAME "CSV_UTIL"
 
@@ -38,6 +39,10 @@
 extern char *xcsv_urlbase;
 extern char *prefer_shortnames;
 
+static double pathdist = 0;
+static double oldlon = 999;
+static double oldlat = 999;
+    
 static int waypt_out_count;
 
 /*********************************************************************/
@@ -131,7 +136,7 @@ csv_stringtrim(const char *string, const char *enclosure, int strip_max)
     if (elen) {
 	while (
 	       (stripped < strip_max) &&
-	       ((size_t) (p2 - p1) > elen) &&
+	       ((size_t) (p2 - p1) >= elen) &&
 	       (strncmp(p1, enclosure, elen) == 0) &&
 	       (strncmp((p2 - elen + 1), enclosure, elen) == 0)) {
 	    p2 -= elen;
@@ -312,6 +317,182 @@ decdir_to_dec(const char * decdir)
     return(rval * sign);
 }
 
+
+/*****************************************************************************
+ * human_to_dec() - convert a "human-readable" lat and/or lon to decimal
+ * usage: human_to_dec( "N 41 09.12' W 085 09.36'", &lat, &lon );
+ *        human_to_dec( "41 9 5.652 N", &lat, &lon );
+ *****************************************************************************/
+
+static void
+human_to_dec( const char *instr, double *outlat, double *outlon )
+{
+    double unk[3] = {999,999,999};
+    double lat[3] = {999,999,999};
+    double lon[3] = {999,999,999};
+    int    latsign = 0;
+    int    lonsign = 0;
+   
+    const char *cur;
+    double *numres = unk;
+    int numind = 0;
+    
+    cur = instr;
+       
+    while ( cur && *cur ) {
+	switch (*cur) {
+	    case 'n': case 's': case 'N': case 'S':
+		if ( unk[0] != 999 ) {
+		    numind = 0;
+		    numres = unk;
+		    lat[0] = unk[0];
+		    lat[1] = unk[1];
+		    lat[2] = unk[2];
+		    unk[0] = unk[1] = unk[2] = 999;
+		}
+		else {
+		    numres = lat;
+		    numind = 0;
+		    lat[0] = lat[1] = lat[2] = 999;
+		}
+		
+		if ( *cur == 'n' || *cur == 'N' ) 
+		    latsign = 1;
+		else
+		    latsign = -1;
+		cur++;
+		break;
+	    case 'w': case 'e': case 'W': case 'E':
+		if ( unk[0] != 999 ) {
+		    numind = 0;
+		    numres = unk;
+		    lon[0] = unk[0];
+		    lon[1] = unk[1];
+		    lon[2] = unk[2];
+		    unk[0] = unk[1] = unk[2] = 999;
+		}
+		else {
+		    numres = lon;
+		    numind = 0;
+		    lon[0] = lon[1] = lon[2] = 999;
+		}
+		
+		if ( *cur == 'e' || *cur == 'E' ) 
+		    lonsign = 1;
+		else
+		    lonsign = -1;
+		cur++;
+		break;
+	    case '1': case '2': case '3': case '4': case '5':
+	    case '6': case '7': case '8': case '9': case '0': case '.':
+		numres[numind] = atof(cur);
+		while (cur && *cur && strchr("1234567890.",*cur)) cur++;
+		break;
+	    default:
+		if (numres[numind] != 999) {
+    		    numind++;
+		    if ( numind > 2 ) {
+		        numres = unk;
+		        numind = 0;
+		    }
+		}
+		cur++;
+		break;      
+	}
+    }
+    
+    if ( outlat ) {
+	if ( lat[0] != 999 ) *outlat = lat[0];
+	if ( lat[1] != 999 ) *outlat += lat[1]/60.0;
+	if ( lat[2] != 999 ) *outlat += lat[2]/3600.0;
+	if ( latsign ) *outlat *= latsign;
+    }
+    if ( outlon ) {
+	if ( lon[0] != 999 ) *outlon = lon[0];
+	if ( lon[1] != 999 ) *outlon += lon[1]/60.0;
+	if ( lon[2] != 999 ) *outlon += lon[2]/3600.0;
+	if ( lonsign ) *outlon *= lonsign;
+    }
+}
+
+/*
+ * dec_to_human - convert decimal degrees to human readable
+ */
+
+void
+dec_to_human( char *buff, const char *format, const char *dirs, double val )
+{
+    char *subformat = NULL;
+    const char *formatptr = NULL;
+    char *percent = NULL;
+    char *type = NULL;
+    
+    int  index = 0; 
+    int  intvals[3] = {0,0,0};
+    double  dblvals[3] = {0,0,0};
+    int  sign = 0;
+    
+    sign = (val < 0) ? 0 : 1;
+    
+    dblvals[0] = fabs(val);
+    intvals[0] = (int)dblvals[0];
+    dblvals[1] = 60*(dblvals[0]-intvals[0]);
+    intvals[1] = (int)dblvals[1];
+    dblvals[2] = 60*(dblvals[1]-intvals[1]);
+    intvals[2] = (int)dblvals[2];
+    
+    subformat = xmalloc( strlen(format)+2);
+    formatptr = format;
+    
+    buff[0] = '\0';
+    
+    while ( formatptr && *formatptr ) {
+	strcpy( subformat, formatptr );
+	percent = strchr( subformat, '%' );
+	if ( percent ) {
+	    type = percent+1+strcspn( percent+1, "cdiouxXeEfgG%" );
+	    *(type+1) = '\0';
+	    switch( *type ) {
+		case 'c':
+		    sprintf( buff+strlen(buff), subformat, dirs[sign] );
+		    break;
+		case 'd':
+		case 'i':
+		case 'o':
+		case 'u':
+		case 'x':
+		case 'X':
+		    if (index>2) fatal(MYNAME ": too many format specifiers\n");
+		    sprintf( buff+strlen(buff), subformat, intvals[index]);
+		    index++;
+		    break;
+		case 'e':
+		case 'E':
+		case 'f':
+		case 'g':
+		case 'G':
+		    if (index>2) fatal(MYNAME ": too many format specifiers\n");
+		    sprintf( buff+strlen(buff), subformat, dblvals[index]);
+		    index++;
+		    break;
+		case '%':
+		    sprintf( buff+strlen(buff), subformat );
+		    break;
+		default:
+		    fatal(MYNAME ": invalid format specifier\n");
+		    break;
+
+	    }
+	}
+	else {
+	    sprintf( buff+strlen(buff), subformat );
+	}
+	formatptr += strlen(subformat);
+    } 
+    
+    xfree(subformat);
+}	
+
 /*****************************************************************************/
 /* xcsv_file_init() - prepare xcsv_file for first use.                       */
 /*****************************************************************************/
@@ -452,6 +633,12 @@ xcsv_parse_val(const char *s, waypoint *wpt, const field_map_t *fmp)
        /* latitude as a 32 bit integer offset */
        wpt->latitude = intdeg_to_dec(atof(s), 1);
     } else
+    if ( strcmp(fmp->key, "LAT_HUMAN_READABLE") == 0) {
+       human_to_dec( s, &wpt->latitude, &wpt->longitude );
+    } else
+    if ( strcmp(fmp->key, "LAT_NMEA") == 0) {
+	wpt->latitude = ddmm2degrees(wpt->latitude);
+    } else
     /* LONGITUDE CONVERSIONS ***********************************************/
     if (strcmp(fmp->key, "LON_DECIMAL") == 0) {
        /* longitude as a pure decimal value */
@@ -466,7 +653,16 @@ xcsv_parse_val(const char *s, waypoint *wpt, const field_map_t *fmp)
        /* longitude as a 32 bit integer offset  */
        wpt->longitude = intdeg_to_dec(atof(s), 0);
     } else
-
+    if ( strcmp(fmp->key, "LON_HUMAN_READABLE") == 0) {
+       human_to_dec( s, &wpt->latitude, &wpt->longitude );
+    } else
+    if ( strcmp(fmp->key, "LON_NMEA") == 0) {
+	wpt->latitude = ddmm2degrees(wpt->longitude);
+    } else
+    /* LAT AND LON CONVERSIONS ********************************************/
+    if ( strcmp(fmp->key, "LATLON_HUMAN_READABLE") == 0) {
+       human_to_dec( s, &wpt->latitude, &wpt->longitude );
+    } else
     /* DIRECTIONS **********************************************************/
     if (strcmp(fmp->key, "LAT_DIR") == 0) {
        /* latitude N/S.  Ignore on input for now */
@@ -505,9 +701,17 @@ xcsv_parse_val(const char *s, waypoint *wpt, const field_map_t *fmp)
        wpt->gc_data.terr = atof(s) * 10;
     } else
     if (strcmp(fmp->key, "GEOCACHE_TYPE") == 0) {
-       /* Geocache Type - TODO */
-       /* wpt->gc_data.type = gs_mktype(s);*/
-
+       /* Geocache Type */
+       wpt->gc_data.type = gs_mktype(s);
+    } else
+    if (strcmp(fmp->key, "GEOCACHE_CONTAINER") == 0) {
+       wpt->gc_data.container = gs_mkcont(s);
+    } else
+    if ( strcmp( fmp->key, "PATH_DISTANCE_MILES") == 0) {
+       /* Ignored on input */
+    } else
+    if ( strcmp( fmp->key, "PATH_DISTANCE_KM") == 0 ) {
+       /* Ignored on input */
     } else {
        warning( MYNAME ": Unknown style directive: %s\n", fmp->key);
     }
@@ -589,6 +793,14 @@ xcsv_data_read(void)
     } while (!feof(xcsv_file.xcsvfp));
 }
 
+static void
+xcsv_resetpathlen()
+{
+    pathdist = 0;
+    oldlat = 999;
+    oldlon = 999;
+}
+
 /*****************************************************************************/
 /* xcsv_waypt_pr() - write output file, handling output conversions          */
 /*                  (the output meat)                                        */
@@ -604,6 +816,13 @@ xcsv_waypt_pr(const waypoint *wpt)
     int i;
     field_map_t *fmp;
     queue *elem, *tmp;
+    
+    if ( oldlon < 900 ) {
+	pathdist += tomiles(gcdist(oldlat*M_PI/180,oldlon*M_PI/180,
+			wpt->latitude*M_PI/180,wpt->longitude*M_PI/180));
+    }
+    oldlon = wpt->longitude;
+    oldlat = wpt->latitude;
 
     if (strcmp(xcsv_file.field_delimiter, "\\w") == 0)
         write_delimiter = " ";
@@ -617,8 +836,7 @@ xcsv_waypt_pr(const waypoint *wpt)
             else
                 shortname = csv_stringclean(wpt->description, xcsv_file.badchars);
         } else {
-            /* no description available */
-            shortname = xstrdup("");
+            /* no shortname available -- let shortname default on output */
         }
     } else{
         shortname = csv_stringclean(wpt->shortname, xcsv_file.badchars);
@@ -628,7 +846,7 @@ xcsv_waypt_pr(const waypoint *wpt)
         if (shortname) {
             description = csv_stringclean(shortname, xcsv_file.badchars);
         } else {
-            description = xstrdup("");
+            /* no description -- let description default on output */
         }
     } else {
         description = csv_stringclean(wpt->description, xcsv_file.badchars);
@@ -667,7 +885,8 @@ xcsv_waypt_pr(const waypoint *wpt)
             sprintf(buff, fmp->printfc, fmp->val);
         } else
         if (strcmp(fmp->key, "SHORTNAME") == 0) {
-            sprintf(buff, fmp->printfc, shortname);
+            sprintf(buff, fmp->printfc, 
+                (shortname && *shortname) ? shortname : fmp->val);
         } else
         if (strcmp(fmp->key, "ANYNAME") == 0) {
             if (wpt->shortname) {
@@ -679,7 +898,7 @@ xcsv_waypt_pr(const waypoint *wpt)
             if (wpt->notes) {
                 anyname = xstrdup(wpt->notes);
             } else
-                anyname = xstrdup("");
+                anyname = xstrdup(fmp->val);
 
             if ((anyname) && (global_opts.synthesize_shortnames)) {
                 anyname = xstrdup(shortname);
@@ -690,10 +909,12 @@ xcsv_waypt_pr(const waypoint *wpt)
             xfree(anyname);
         } else
         if (strcmp(fmp->key, "DESCRIPTION") == 0) {
-            sprintf(buff, fmp->printfc, description);
+            sprintf(buff, fmp->printfc, 
+                (description && *description) ? description : fmp->val);
         } else
         if (strcmp(fmp->key, "NOTES") == 0) {
-	    sprintf(buff, fmp->printfc, wpt->notes? wpt->notes : "");
+	    sprintf(buff, fmp->printfc, 
+	        (wpt->notes && *wpt->notes) ? wpt->notes : fmp->val);
         } else
         if (strcmp(fmp->key, "URL") == 0) {
 	    int off = 0;
@@ -704,13 +925,16 @@ xcsv_waypt_pr(const waypoint *wpt)
 	    if (wpt->url)
 		sprintf(buff + off, fmp->printfc, wpt->url);
 	    else
-		strcpy(buff, "\"\"");
+		strcpy(buff, (fmp->val && *fmp->val) ? fmp->val : "\"\"");
         } else
         if (strcmp(fmp->key, "URL_LINK_TEXT") == 0) {
-            sprintf(buff, fmp->printfc, NONULL(wpt->url_link_text));
+            sprintf(buff, fmp->printfc, 
+                (wpt->url_link_text && *wpt->url_link_text) ? wpt->url_link_text : fmp->val);
         } else
         if (strcmp(fmp->key, "ICON_DESCR") == 0) {
-            sprintf(buff, fmp->printfc, NONULL(wpt->icon_descr));
+            sprintf(buff, fmp->printfc, 
+                (wpt->icon_descr && *wpt->icon_descr) ? 
+                wpt->icon_descr : fmp->val);
         } else
 
         /* LATITUDE CONVERSION***********************************************/
@@ -734,6 +958,12 @@ xcsv_waypt_pr(const waypoint *wpt)
             sprintf(buff, fmp->printfc,
               dec_to_intdeg(wpt->latitude, 1));
         } else
+	if (strcmp(fmp->key, "LAT_HUMAN_READABLE") == 0) {
+	    dec_to_human( buff, fmp->printfc, "SN", wpt->latitude );
+	} else
+	if (strcmp(fmp->key, "LAT_NMEA") == 0) {
+		sprintf(buff, fmp->printfc, degrees2ddmm(wpt->latitude));
+	} else
 
         /* LONGITUDE CONVERSIONS*********************************************/
         if (strcmp(fmp->key, "LON_DECIMAL") == 0) {
@@ -757,6 +987,18 @@ xcsv_waypt_pr(const waypoint *wpt)
             sprintf(buff, fmp->printfc,
               dec_to_intdeg(wpt->longitude, 0));
         } else
+	if (strcmp(fmp->key, "LON_HUMAN_READABLE") == 0) {
+	    dec_to_human( buff, fmp->printfc, "WE", wpt->longitude );
+	} else
+	if (strcmp(fmp->key, "LATLON_HUMAN_READABLE") == 0) {
+	    dec_to_human( buff, fmp->printfc, "SN", wpt->latitude );
+	    if ( !isspace(buff[strlen(buff)])) strcat( buff, " " );
+	    dec_to_human( buff+strlen(buff), fmp->printfc, "WE", 
+			    wpt->longitude );
+	} else
+	if (strcmp(fmp->key, "LON_NMEA") == 0) {
+		sprintf(buff, fmp->printfc, degrees2ddmm(wpt->longitude));
+	} else
 
         /* DIRECTIONS *******************************************************/
         if (strcmp(fmp->key, "LAT_DIR") == 0) {
@@ -781,6 +1023,16 @@ xcsv_waypt_pr(const waypoint *wpt)
             sprintf(buff, fmp->printfc,
               wpt->altitude);
         } else
+		
+        /* DISTANCE CONVERSIONS**********************************************/
+	if (strcmp(fmp->key, "PATH_DISTANCE_MILES") == 0) {
+            /* path (route/track) distance in miles */
+            sprintf( buff, fmp->printfc, pathdist );
+	} else
+	if (strcmp(fmp->key, "PATH_DISTANCE_KM") == 0) {
+            /* path (route/track) distance in  */
+            sprintf( buff, fmp->printfc, pathdist * 5280*12*2.54/100/1000 );
+	} else
 
         /* TIME CONVERSIONS**************************************************/
         if (strcmp(fmp->key, "EXCEL_TIME") == 0) {
@@ -801,9 +1053,13 @@ xcsv_waypt_pr(const waypoint *wpt)
             /* Geocache Terrain as a double */
             sprintf(buff, fmp->printfc, wpt->gc_data.terr / 10.0);
         } else
-        if (strcmp(fmp->key, "GEOCACHE_TYPE") == 0) {
-            /* Geocache Type TODO */
-            sprintf(buff, fmp->printfc, "Geocache");
+        if (strcmp(fmp->key, "GEOCACHE_CONTAINER") == 0) {
+            /* Geocache Container */
+            sprintf(buff, fmp->printfc, gs_get_container(wpt->gc_data.container));
+	} else
+	if (strcmp(fmp->key, "GEOCACHE_TYPE") == 0) {
+            /* Geocache Type */
+            sprintf(buff, fmp->printfc, gs_get_cachetype(wpt->gc_data.type));
         } else {
            /* this should probably never happen */
         }
@@ -860,8 +1116,8 @@ xcsv_data_write(void)
     }
 
     waypt_disp_all(xcsv_waypt_pr);
-    route_disp_all(xcsv_noop,xcsv_noop,xcsv_waypt_pr);
-    track_disp_all(xcsv_noop,xcsv_noop,xcsv_waypt_pr);
+    route_disp_all(xcsv_resetpathlen,xcsv_noop,xcsv_waypt_pr);
+    track_disp_all(xcsv_resetpathlen,xcsv_noop,xcsv_waypt_pr);
 
     /* output epilogue lines, if any. */
     QUEUE_FOR_EACH(&xcsv_file.epilogue, elem, tmp) {
