@@ -1,7 +1,7 @@
 /*
     Access GPX data files.
 
-    Copyright (C) 2002 Robert Lipe, robertlipe@usa.net
+    Copyright (C) 2002, 2003, 2004 Robert Lipe, robertlipe@usa.net
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -77,6 +77,7 @@ typedef enum {
 	tt_wpt_time,
 	tt_wpt_type,
 	tt_wpt_urlname,
+	tt_cache,
 	tt_cache_name,
 	tt_cache_container,
 	tt_cache_type,
@@ -104,6 +105,7 @@ typedef enum {
 	tt_trk_desc,
 	tt_trk_name,
 	tt_trk_trkseg,
+	tt_trk_number,
 	tt_trk_trkseg_trkpt,
 	tt_trk_trkseg_trkpt_cmt,
 	tt_trk_trkseg_trkpt_name,
@@ -145,6 +147,7 @@ tag_mapping tag_path_map[] = {
 	{ tt_wpt_urlname, 0, "/gpx/wpt/urlname" },
 	{ tt_wpt_sym, 0, "/gpx/wpt/sym" },
 	{ tt_wpt_type, 1, "/gpx/wpt/type" },
+	{ tt_cache, 1, "/gpx/wpt/groundspeak:cache" },
 	{ tt_cache_name, 1, "/gpx/wpt/groundspeak:cache/groundspeak:name" },
 	{ tt_cache_container, 1, "/gpx/wpt/groundspeak:cache/groundspeak:container" },
 	{ tt_cache_type, 1, "/gpx/wpt/groundspeak:cache/groundspeak:type" },
@@ -173,6 +176,7 @@ tag_mapping tag_path_map[] = {
 	{ tt_trk_name, 0, "/gpx/trk/name" },
 	{ tt_trk_desc, 0, "/gpx/trk/desc" },
 	{ tt_trk_trkseg, 0, "/gpx/trk/trkseg" },
+	{ tt_trk_number, 0, "/gpx/trk/number" },
 	{ tt_trk_trkseg_trkpt, 0, "/gpx/trk/trkseg/trkpt" },
 	{ tt_trk_trkseg_trkpt_ele, 0, "/gpx/trk/trkseg/trkpt/ele" },
 	{ tt_trk_trkseg_trkpt_time, 0, "/gpx/trk/trkseg/trkpt/time" },
@@ -274,6 +278,19 @@ tag_cache_desc(const char ** attrv)
 	}
 }
 
+static void
+tag_gs_cache(const char **attrv)
+{
+	const char **avp;
+
+	cache_descr_is_html = 0;
+	for (avp = &attrv[0]; *avp; avp+=2) {
+		if (strcmp(avp[0], "id") == 0) {
+				wpt_tmp->gc_data.id = atoi(avp[1]);
+		}
+	}
+}
+
 static void
 start_something_else(const char *el, const char **attrv)
 {
@@ -427,6 +444,9 @@ gpx_start(void *data, const char *el, const char **attr)
 	case tt_unknown:
 		start_something_else(el, attr);
 		return;
+	case tt_cache:
+		tag_gs_cache(attr);
+		break;
 	case tt_cache_log_wpt:
 		if (opt_logpoint)
 			tag_log_wpt(attr);
@@ -465,10 +485,10 @@ gs_container_mapping{
 	{ gc_micro, "Micro" },
 	{ gc_regular, "Regular" },
 	{ gc_large, "Large" },
+	{ gc_small, "Small" },
 	{ gc_virtual, "Virtual" }
 };
 
-static
 geocache_type
 gs_mktype(char *t)
 {
@@ -483,7 +503,20 @@ gs_mktype(char *t)
 	return gt_unknown;
 }
 
-static
+const char *
+gs_get_cachetype(geocache_type t)
+{
+	int i;
+	int sz = sizeof(gs_type_map) / sizeof(gs_type_map[0]);
+
+	for (i = 0; i < sz; i++) {
+		if (t == gs_type_map[i].type) {
+			return gs_type_map[i].name;
+		}
+	}
+	return "Unknown";
+}
+
 geocache_container
 gs_mkcont(char *t)
 {
@@ -498,7 +531,20 @@ gs_mkcont(char *t)
 	return gc_unknown;
 }
 
-static
+const char *
+gs_get_container(geocache_container t)
+{
+	int i;
+	int sz = sizeof(gs_container_map) / sizeof(gs_container_map[0]);
+
+	for (i = 0; i < sz; i++) {
+		if (t == gs_container_map[i].type) {
+			return gs_container_map[i].name;
+		}
+	}
+	return "Unknown";
+}
+
 time_t 
 xml_parse_time( char *cdatastr ) 
 {
@@ -677,6 +723,9 @@ gpx_end(void *data, const char *el)
 	case tt_trk_desc:
 		trk_head->rte_desc = xstrdup(cdatastrp);
 		break;
+	case tt_trk_number:
+		trk_head->rte_num = atoi(cdatastrp);
+		break;
 
 	/*
 	 * Items that are actually in multiple categories.
@@ -730,7 +779,7 @@ gpx_end(void *data, const char *el)
 void
 gpx_rd_init(const char *fname)
 {
-	fatal(MYNAME ": This build excluded GPX support becuase expat was not installed.\n");
+	fatal(MYNAME ": This build excluded GPX support because expat was not installed.\n");
 }
 
 #else /* NO_EXPAT */
@@ -799,7 +848,7 @@ gpx_rd_init(const char *fname)
 	cdatastr = vmem_alloc(1, 0);
 	*((char *)cdatastr.mem) = '\0';
 
-	/* We don't use xstrdup here becuase we' know we don't free
+	/* We don't use xstrdup here because we' know we don't free
 	 * this across reads and we unlock the safety belt from the 
 	 * leak tester.
 	 */
@@ -1014,11 +1063,11 @@ fprint_xml_chain( xml_tag *tag, const waypoint *wpt )
 						"groundspeak:exported" );
 			}
 			fprintf( ofd, "</%s>", tag->tagname);
-			if ( tag->parentcdata ) {
-				tmp_ent = xml_entitize(tag->parentcdata);
-				fprintf(ofd, "%s", tmp_ent );
-				xfree(tmp_ent);
-			}
+		}
+		if ( tag->parentcdata ) {
+			tmp_ent = xml_entitize(tag->parentcdata);
+			fprintf(ofd, "%s", tmp_ent );
+			xfree(tmp_ent);
 		}
 		tag = tag->sibling;	
 	}
@@ -1079,7 +1128,7 @@ gpx_waypt_pr(const waypoint *waypointp)
 				  mkshort(mkshort_handle, odesc) : 
 				  waypointp->shortname;
 
-	fprintf(ofd, "<wpt lat=\"%lf\" lon=\"%lf\">\n",
+	fprintf(ofd, "<wpt lat=\"%0.9lf\" lon=\"%0.9lf\">\n",
 		waypointp->latitude,
 		waypointp->longitude);
 	if (waypointp->creation_time) {
@@ -1215,7 +1264,7 @@ gpx_write(void)
 
 	fprintf(ofd, "<?xml version=\"1.0\"?>\n");
 	fprintf(ofd, "<gpx\n version=\"1.0\"\n");
-	fprintf(ofd, "creator=\"GPSBabel - http://gpsbabel.sourceforge.net\"\n");
+	fprintf(ofd, "creator=\"GPSBabel - http://www.gpsbabel.org\"\n");
 	fprintf(ofd, "xmlns:xsi=\"http://www.w3.org/2001/XMLSchema-instance\"\n");
 	fprintf(ofd, "xmlns=\"http://www.topografix.com/GPX/1/0\"\n");
 	fprintf(ofd, "xsi:schemaLocation=\"%s\">\n", xsi_schema_loc ? xsi_schema_loc : DEFAULT_XSI_SCHEMA_LOC);
@@ -1223,8 +1272,8 @@ gpx_write(void)
 	gpx_write_time( now, "time" );
 	waypt_compute_bounds(&bounds);
 	if (bounds.max_lat  > -360) {
-		fprintf(ofd, "<bounds minlat=\"%f\" minlon =\"%f\" "
-			       "maxlat=\"%f\" maxlon=\"%f\" />\n",
+		fprintf(ofd, "<bounds minlat=\"%0.9f\" minlon =\"%0.9f\" "
+			       "maxlat=\"%0.9f\" maxlon=\"%0.9f\" />\n",
 			       bounds.min_lat, bounds.min_lon, 
 			       bounds.max_lat, bounds.max_lon);
 	}
@@ -1239,13 +1288,18 @@ static
 arglist_t gpx_args[] = {
 	{ "gsshortnames", &gsshortnames, 
 		"Prefer shorter descriptions from Groundspeak files",
-		ARGTYPE_BOOL },
-	{ "snlen", &snlen, "Length of generated shortnames", ARGTYPE_INT },
+		NULL, ARGTYPE_BOOL },
+	{ "snlen", &snlen, "Length of generated shortnames", 
+		NULL, ARGTYPE_INT },
 	{ "suppresswhite", &suppresswhite, 
-		"Suppress whitespace in generated shortnames", ARGTYPE_BOOL },
-	{ "logpoint", &opt_logpoint, "Create waypoints from geocache log entries", ARGTYPE_BOOL },
-	{ "urlbase", &urlbase, "Base URL for link tag in output", ARGTYPE_STRING},
-	{ 0, 0, 0, 0 }
+		"Suppress whitespace in generated shortnames", 
+		NULL, ARGTYPE_BOOL },
+	{ "logpoint", &opt_logpoint, 
+		"Create waypoints from geocache log entries", 
+		NULL, ARGTYPE_BOOL },
+	{ "urlbase", &urlbase, "Base URL for link tag in output", 
+		NULL, ARGTYPE_STRING},
+	{ 0, 0, 0, 0, 0 }
 };
 
 ff_vecs_t gpx_vecs = {
@@ -1256,5 +1310,6 @@ ff_vecs_t gpx_vecs = {
 	gpx_wr_deinit,	
 	gpx_read,
 	gpx_write,
+	NULL, 
 	gpx_args,
 };
