@@ -129,7 +129,7 @@ typedef struct GPS_SWay
     int32  colour;
     char   cc[2];
     UC     wpt_class;
-    int32  alt;
+    float  alt;
     char   city[24];
     char   state[2];
     char   name[30];
@@ -177,6 +177,7 @@ double gps_save_lon;
 extern int32  gps_save_id;
 extern double gps_save_version;
 extern char   gps_save_string[GPS_ARB_LEN];
+extern int gps_is_usb;
 
 
 extern struct COMMANDDATA COMMAND_ID[2];
