@@ -1,7 +1,7 @@
 /*
     Communicate Thales/Magellan serial protocol.
 
-    Copyright (C) 2002 Robert Lipe, robertlipe@usa.net
+    Copyright (C) 2002, 2003, 2004 Robert Lipe, robertlipe@usa.net
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -35,11 +35,13 @@ static void termwrite(char *obuf, int size);
 static void mag_readmsg(void);
 static void mag_handon(void);
 static void mag_handoff(void);
-static void *mkshort_handle;
-static char *deficon;
-static char *bs;
-static char *noack;
+static void *mkshort_handle = NULL;
+static char *deficon = NULL;
+static char *bs = NULL;
+static char *noack = NULL;
+static char *nukewpt = NULL;
 static int route_out_count;
+static int waypoint_read_count;
 
 typedef enum {
 	mrs_handoff = 0,
@@ -114,6 +116,7 @@ static icon_mapping_t map330_icon_table[] = {
 	{ "e", "airport" },
 	{ "f", "amusement park" },
 	{ "g", "ATM" },
+	{ "g", "Bank" },
 	{ "h", "auto repair" },
 	{ "i", "boating" },
 	{ "j", "camping" },
@@ -149,7 +152,7 @@ static icon_mapping_t map330_icon_table[] = {
 	{ "an", "winery" },
 	{ "ao", "wreck" },
 	{ "ap", "zoo" },
-	{ "ah", "Virtual cache"}, 	/* Binos: becuase you "see" them. */
+	{ "ah", "Virtual cache"}, 	/* Binos: because you "see" them. */
 	{ "ak", "Micro-Cache" },	/* Looks like a film canister. */
 	{ "an", "Multi-Cache"}, 	/* Winery: grapes 'coz they "bunch" */
 	{ "s",  "Unknown Cache"}, 	/* 'Suprise' cache: use a target. */
@@ -430,6 +433,11 @@ retry:
 	} 
 	if (strncmp(ibuf, "$PMGNWPT,", 7) == 0) {
 		waypoint *wpt = mag_wptparse(ibuf);
+		waypoint_read_count++;
+		if (global_opts.verbose_status) {
+			waypt_status_disp(waypoint_read_count, 
+					waypoint_read_count);
+		}
 		switch (global_opts.objective)
 		{
 			case wptdata:
@@ -555,10 +563,13 @@ try_as_file:
 	}
 
 	GetCommTimeouts (comport, &timeout);
-	timeout.ReadIntervalTimeout = 100;
-	timeout.ReadTotalTimeoutMultiplier = 100;
-	timeout.ReadTotalTimeoutConstant = 100;
-	timeout.WriteTotalTimeoutMultiplier = 100;
+	/* We basically do single character reads and simulate line input
+	 * mode, so these values are kind of fictional.
+	 */
+	timeout.ReadIntervalTimeout = 1000;
+	timeout.ReadTotalTimeoutMultiplier = 1000;
+	timeout.ReadTotalTimeoutConstant = 1000;
+	timeout.WriteTotalTimeoutMultiplier = 1000;
 	timeout.WriteTotalTimeoutConstant = 1000;
 	if (!SetCommTimeouts (comport, &timeout)) {
 		xCloseHandle (comport);
@@ -698,23 +709,26 @@ termwrite(char *obuf, int size)
  */
 static
 arglist_t mag_sargs[] = {
-	{"baud", &bs, "Numeric value of bitrate (baud=4800)", ARGTYPE_INT },
+	{"baud", &bs, "Numeric value of bitrate (baud=4800)", NULL,
+		ARGTYPE_INT },
 	{"noack", &noack, "Suppress use of handshaking in name of speed",
-		ARGTYPE_BOOL},
-	{"deficon", &deficon, "Default icon name", ARGTYPE_STRING },
-	{0, 0, 0, 0}
+		NULL, ARGTYPE_BOOL},
+	{"deficon", &deficon, "Default icon name", NULL, ARGTYPE_STRING },
+	{"nukewpt", &nukewpt, "Delete all waypoints", NULL, ARGTYPE_BOOL },
+	{0, 0, 0, 0, 0}
 };
 
 static
 arglist_t mag_fargs[] = {
-	{"deficon", &deficon, "Default icon name", ARGTYPE_STRING },
-	{0, 0, 0, 0}
+	{"deficon", &deficon, "Default icon name", NULL, ARGTYPE_STRING },
+	{0, 0, 0, 0, 0}
 };
 
 static void
 mag_rd_init(const char *portname)
 {
 	time_t now, later;
+	waypoint_read_count = 0;
 
 	if (bs) {
 		bitrate=atoi(bs);
@@ -756,6 +770,15 @@ mag_rd_init(const char *portname)
 		mag_writemsg("PMGNCMD,NMEAOFF");
 		ignore_unable = 0;
 	}
+	if (nukewpt) {
+		/* The unit will send us an "end" message upon completion */
+		mag_writemsg("PMGNCMD,DELETE,WAYPOINT");
+		mag_readmsg();
+		if (!found_done) {
+			fatal(MYNAME ": Unexpected response to waypoint delete command.\n");
+		}
+		found_done = 0;
+	}
 
 	QUEUE_INIT(&rte_wpt_tmp);
 
@@ -1369,6 +1392,7 @@ ff_vecs_t mag_svecs = {
 	mag_deinit,	
 	mag_read,
 	mag_write,
+	NULL, 
 	mag_sargs
 };
 
@@ -1380,5 +1404,6 @@ ff_vecs_t mag_fvecs = {
 	mag_deinit,	
 	mag_read,
 	mag_write,
+	NULL, 
 	mag_fargs
 };
