@@ -40,22 +40,22 @@ static int track_out_count;
 static int route_out_count;
 static int route_wpt_count;
 
-static char *snlenopt;
-static char *snwhiteopt;
-static char *snupperopt;
-static char *snuniqueopt;
+static char *snlenopt = NULL;
+static char *snwhiteopt = NULL;
+static char *snupperopt = NULL;
+static char *snuniqueopt = NULL;
 
 static
 arglist_t ozi_args[] = {
 	{"snlen", &snlenopt, "Max synthesized shortname length",
-		ARGTYPE_INT},
+		NULL, ARGTYPE_INT},
 	{"snwhite", &snwhiteopt, "(0/1) Allow whitespace synth. shortnames",
-		ARGTYPE_BOOL},
+		NULL, ARGTYPE_BOOL},
 	{"snupper", &snupperopt, "(0/1) UPPERCASE synth. shortnames",
-	        ARGTYPE_BOOL},
+	        NULL, ARGTYPE_BOOL},
 	{"snunique", &snuniqueopt, "(0/1) Make synth. shortnames unique",
-		ARGTYPE_BOOL},
-	{0, 0, 0, 0}
+		NULL, ARGTYPE_BOOL},
+	{0, 0, 0, 0, 0}
 };
 
 gpsdata_type ozi_objective;
@@ -79,7 +79,7 @@ ozi_openfile(char *fname) {
     }
 
     /* allocate more than enough room for new filename */
-    tmpname = xcalloc(1, strlen(fname) +
+    tmpname = (char *) xcalloc(1, strlen(fname) +
                          strlen(buff) +
                          strlen(ozi_extensions[ozi_objective]) +
                          2); /* . (dot) plus null term */
@@ -550,7 +550,7 @@ data_read(void)
         }
         if ((strlen(buff)) && (strstr(buff, ",") != NULL)) {
 
-            wpt_tmp = xcalloc(sizeof(*wpt_tmp), 1);
+            wpt_tmp = waypt_new();
 
             /* data delimited by commas, possibly enclosed in quotes.  */
             s = buff;
@@ -698,5 +698,6 @@ ff_vecs_t ozi_vecs = {
     wr_deinit,
     data_read,
     data_write,
+    NULL,
     ozi_args
 };
