@@ -56,8 +56,9 @@ char *snlen;
 
 static
 arglist_t psit_args[] = {
-/*	{"snlen", &snlen, "Length of generated shortnames", ARGTYPE_INT }, */
-	{0, 0, 0, 0}
+/*	{"snlen", &snlen, "Length of generated shortnames", 
+ 	NULL, ARGTYPE_INT }, */
+	{0, 0, 0, 0, 0}
 };
 
 /* Taken from PsiTrex 1.13 */
@@ -288,7 +289,7 @@ psit_waypoint_r(FILE *psit_file, waypoint **wpt)
 	double	psit_depth = unknown_alt;
 
 	if (strlen(psit_current_token) > 0) {
-		thisWaypoint = xcalloc(sizeof(*thisWaypoint), 1);
+		thisWaypoint = waypt_new();
 
 		thisWaypoint->latitude = atof(psit_current_token);
 
@@ -414,7 +415,7 @@ psit_route_r(FILE *psit_file, route_head **rte)
 
 	while (psit_isKnownToken(psit_current_token) != 0) {
 		if (strlen(psit_current_token) > 0) {
-			thisWaypoint = xcalloc(sizeof(*thisWaypoint), 1);
+			thisWaypoint = waypt_new();
 
 			thisWaypoint->latitude = atof(psit_current_token);
 
@@ -545,7 +546,7 @@ psit_track_r(FILE *psit_file, route_head **trk)
 
 	while (psit_isKnownToken(psit_current_token) != 0) {
 		if (strlen(psit_current_token) > 0) {
-			thisWaypoint = xcalloc(sizeof(*thisWaypoint), 1);
+			thisWaypoint = waypt_new();
 
 			thisWaypoint->latitude = atof(psit_current_token);
 
@@ -804,5 +805,6 @@ ff_vecs_t psit_vecs = {
 	psit_wr_deinit,
 	psit_read,
 	psit_write,
+	NULL, 
 	psit_args
 };
