@@ -152,7 +152,7 @@ xstrndup(const char *str, size_t sz)
 		newlen = sz;
 	}
 
-	newstr = xmalloc(newlen + 1);
+	newstr = (char *) xmalloc(newlen + 1);
 	memcpy(newstr, str, newlen);    
 	newstr[newlen] = 0;
 
@@ -178,7 +178,7 @@ xstrndupt(const char *str, size_t sz)
 		newlen = sz;
 	}
 
-	newstr = xmalloc(newlen + 1);
+	newstr = (char *) xmalloc(newlen + 1);
 	memcpy(newstr, str, newlen);
 	newstr[newlen] = '\0';
 	rtrim(newstr);
@@ -193,7 +193,7 @@ XREALLOC(void *p, size_t s, DEBUG_PARAMS )
 xrealloc(void *p, size_t s)
 #endif
 {
-	char *o = realloc(p,s);
+	char *o = (char *) realloc(p,s);
 #ifdef DEBUG_MEM
 	debug_mem_output( "realloc, %x, %x, %x, %s, %d\n", 
 			o, p, s, file, line );
@@ -211,20 +211,20 @@ xrealloc(void *p, size_t s)
 */
 char *
 #ifdef DEBUG_MEM
-XSTRAPPEND(char *src, const char *new, DEBUG_PARAMS)
+XSTRAPPEND(char *src, const char *newd, DEBUG_PARAMS)
 #else
-xstrappend(char *src, const char *new)
+xstrappend(char *src, const char *newd)
 #endif
 {
 	size_t newsz;
 
 	if (!src) {
-		return xxstrdup(new, file, line);
+		return xxstrdup(newd, file, line);
 	}
 
-	newsz = strlen(src) + strlen(new) + 1;
+	newsz = strlen(src) + strlen(newd) + 1;
 	src = xxrealloc(src, newsz, file, line);
-	strcat(src, new);
+	strcat(src, newd);
 
 	return src;
 }
@@ -255,6 +255,26 @@ xfopen(const char *fname, const char *type, const char *errtxt)
 	return f;
 }
 
+void
+xfprintf(const char *errtxt, FILE *stream, const char *format, ...)
+{
+	va_list ap;
+	va_start(ap, format);
+	if (vfprintf(stream, format, ap) < 0) {
+		fatal("%s writing output file.  Error was '%s'.\n",
+				errtxt, strerror(errno));
+	}
+	va_end(ap);
+}
+
+void
+xfputs(const char *errtxt, const char *s, FILE *stream)
+{
+	if (fputs(s, stream) < 0) {
+		fatal("%s Writing output file.  Error was '%s'.\n",
+				errtxt, strerror(errno));
+	}
+}
 
 /* 
  * Duplicate a pascal string into a normal C string.
@@ -765,14 +785,19 @@ char * str_utf8_to_cp1252( const char * str )
 
 char * str_utf8_to_ascii( const char * str )
 {
-	char *result = xstrdup( str );
-	char *cur = result;
+	char *result;
+	char *cur;
+
+	if (!str) return NULL;
+
+	result = xstrdup( str );
+	cur = result;
 	
 	while ( cur && *cur ) {
 		if ( *cur & 0x80 ) {
 			int bytes;
 			int value;
-			char *strvalue = "";
+			char *strvalue = NULL;
 			utf8_to_int( cur, &bytes, &value );
 			switch (value) {
 				case 0x2026: strvalue = "..."; break;
@@ -1075,3 +1100,57 @@ char * html_entitize(const char * str)
 {
 	return entitize(str, 1);
 }
+
+/*
+ * xml_tag utilities
+ */
+
+xml_tag *xml_next( xml_tag *root, xml_tag *cur )
+{
+	if ( cur->child ) {
+		cur = cur->child;
+	}
+	else if ( cur->sibling ) {
+		cur = cur->sibling;
+	}
+	else {
+		cur = cur->parent;
+		if ( cur == root ) {
+			cur = NULL;
+		}
+		if ( cur ) {
+			cur = cur->sibling;
+		}
+	}
+	return cur;
+}
+
+xml_tag *xml_findnext( xml_tag *root, xml_tag *cur, char *tagname ) 
+{
+	xml_tag *result = cur;
+	do {
+		result = xml_next( root, result );
+	} while ( result && case_ignore_strcmp( result->tagname, tagname ));
+	return result;
+}
+
+xml_tag *xml_findfirst( xml_tag *root, char *tagname )
+{
+	return xml_findnext( root, root, tagname );
+}
+
+char *xml_attribute( xml_tag *tag, char *attrname ) 
+{
+	char *result = NULL;
+	if ( tag->attributes ) {
+		char **attr = tag->attributes;
+		while ( attr && *attr ) {
+			if ( 0 == case_ignore_strcmp( *attr, attrname )) {
+				result = attr[1];
+				break;
+			}
+			attr+=2;
+		}
+	}
+	return result;
+}
