@@ -37,7 +37,7 @@ waypoint *
 waypt_dupe(const waypoint *wpt) 
 {
 	waypoint * tmp;
-	tmp = xcalloc(sizeof *wpt, 1);
+	tmp = waypt_new();
 	memcpy(tmp, wpt, sizeof(waypoint));
 
 	if (wpt->shortname)
@@ -52,6 +52,14 @@ waypt_dupe(const waypoint *wpt)
 		tmp->url_link_text = xstrdup(wpt->url_link_text);
 	if (wpt->icon_descr && wpt->icon_descr_is_dynamic)
 		tmp->icon_descr = xstrdup(wpt->icon_descr);
+	if (wpt->gc_data.desc_short.utfstring) {
+		tmp->gc_data.desc_short.utfstring = 
+			xstrdup(tmp->gc_data.desc_short.utfstring);
+	}
+	if (wpt->gc_data.desc_long.utfstring) {
+		tmp->gc_data.desc_long.utfstring = 
+			xstrdup(tmp->gc_data.desc_long.utfstring);
+	}
 	/*
 	 * It's important that this duplicated waypoint not appear
 	 * on the master Q.
@@ -66,21 +74,27 @@ void
 waypt_add(waypoint *wpt)
 {
 	ENQUEUE_TAIL(&waypt_head, &wpt->Q);
+	waypt_ct++;
 
 	/*
 	 * Some input may not have one or more of these types so we
 	 * try to be sure that we have these fields even if just by
 	 * copying them from elsewhere.
 	 */
+
 	if (wpt->shortname == NULL) {
 		if (wpt->description) {
 			wpt->shortname = xstrdup(wpt->description);
+		} else if (wpt->notes) {
+			wpt->shortname = xstrdup(wpt->notes);
 		} else {
-			if (wpt->notes) {
-				wpt->shortname = xstrdup(wpt->notes);
-			}
+		/* Last ditch:  make up a name */
+			char cbuf[10];
+			snprintf(cbuf, sizeof(cbuf), "WPT%03d", waypt_ct);
+			wpt->shortname = xstrdup(cbuf);
 		}
 	}
+
 	if (wpt->description == NULL || strlen(wpt->description) == 0) {
 		if (wpt->description)
 			xfree(wpt->description);
@@ -92,7 +106,6 @@ waypt_add(waypoint *wpt)
 			}
 		}
 	}
-	waypt_ct++;
 }
 
 void
@@ -110,7 +123,7 @@ waypt_new(void)
 {
 	waypoint *wpt;
 
-	wpt = xcalloc(sizeof (*wpt), 1);
+	wpt = (waypoint *) xcalloc(sizeof (*wpt), 1);
 	wpt->altitude = unknown_alt;
 
 	return wpt;
@@ -131,21 +144,30 @@ waypt_disp(const waypoint *wpt)
 	}
 	printposn(wpt->latitude,1);
 	printposn(wpt->longitude,0);
-	
-	tmpdesc = str_utf8_to_ascii( wpt->description);
-	printf("%s/%s", 
-		global_opts.synthesize_shortnames ? 
+
+	if ( wpt->description ) {	
+		tmpdesc = str_utf8_to_ascii( wpt->description);
+		printf("%s/%s", 
+			global_opts.synthesize_shortnames ? 
 			mkshort(mkshort_handle, tmpdesc) : 
 				wpt->shortname, 
 				tmpdesc);
-	if ( tmpdesc ) 
-		xfree(tmpdesc);
+		if ( tmpdesc ) 
+			xfree(tmpdesc);
+	}
 
 	if (wpt->altitude != unknown_alt)
 		printf(" %f", wpt->altitude);
 	printf("\n");
 }
 
+void
+waypt_status_disp(int total_ct, int myct)
+{
+	fprintf(stdout, "%d/%d/%d\r", myct*100/total_ct, myct, total_ct);
+	fflush(stdout);
+}
+
 void
 waypt_disp_all(waypt_cb cb)
 {
@@ -157,8 +179,7 @@ waypt_disp_all(waypt_cb cb)
 		waypointp = (waypoint *) elem;
 		if (global_opts.verbose_status) {
 			i++;
-			fprintf(stdout, "%d/%d/%d\r", i*100/waypt_ct, i, waypt_ct);
-			fflush(stdout);
+			waypt_status_disp(waypt_ct, i);
 		}
 		(*cb) (waypointp);
 	}
@@ -237,6 +258,12 @@ waypt_free( waypoint *wpt )
 	if (wpt->gpx_extras) {
 		free_gpx_extras(wpt->gpx_extras);
 	}
+	if (wpt->gc_data.desc_short.utfstring) {
+		xfree(wpt->gc_data.desc_short.utfstring);
+	}
+	if (wpt->gc_data.desc_long.utfstring) {
+		xfree(wpt->gc_data.desc_long.utfstring);
+	}
 	xfree(wpt);	
 }
 
